\subsection{Option handling}

You know the drill...

\begin{code}

module Options where
import System (getArgs)
import System.Console.GetOpt

getOptions  = getArgs >>= (return . getOpt Permute options) 

usage :: [String] -> a
usage errs = error $ usageInfo (concat errs
  ++ "Usage: rbr [-k <k>] [-p <parser>] [-w <writer>] [FILE..]") options

type MPars = (Double,Double,Double) -- stdevs thres, 1-elim, stringency

data MaskWith = Lower | Ns
data Args = File FilePath | Kval !Int | Thr !Double | Str !Double | NoStats
          | Dist | Verb | Gap !Int | Library FilePath | Server | MaskLower | Skip !Int
          deriving Show

options :: [OptDescr Args]
options = [Option ['k'] ["word-size"] (ReqArg (Kval . read) "Int") "Word size"
          ,Option [] ["sparse-keys"] (ReqArg (Skip . read) "Int") "Skip length"
--          ,Option [] ["shape"] (ReqArg ...) "Key shape"
          ,Option ['N'] ["no-stats"]   (NoArg NoStats) "Omit statistical masking"
          ,Option ['g'] ["gapclosing"] (ReqArg (Gap . read) "Int") "Close gaps"
          ,Option ['s'] ["stringency"] (ReqArg (Str . read) "Float") "Mask stringecy"
          ,Option ['t'] ["threshold"]  (ReqArg (Thr . read) "Float") "Mask threshold"

          ,Option ['D'] ["distribution"] (NoArg Dist) "Distribution output"
          ,Option ['l'] ["library"] (ReqArg (Library) "file") "Mask words from library"
          ,Option ['L'] ["lower-case"] (NoArg MaskLower) "Mask using lower case letters"
          ,Option [] ["server"] (NoArg Server) "Run in server mode"
          ,Option ['o'] ["output-file"] (ReqArg File "file") "Output file"
	  ,Option ['v'] ["verbose"] (NoArg Verb) "progress messages"] 

data Opts = Opts { kval :: Int, ofile :: Maybe FilePath
                 , stats :: Bool
                 , thresh, string  :: Double
                 , gap :: Int
                 , lower :: MaskWith
                 , distr, verb, server :: Bool
                 , lib :: Maybe FilePath
                 , skip :: Int
                 }

defaultopts = Opts 15 Nothing True 6.0 1.5 0 Ns False False False Nothing 0

parseargs :: [Args] -> Opts
parseargs = foldr p1 defaultopts
    where p1 (File f) s = s {ofile = Just f}
          p1 (Kval k) s = s {kval = k}
          p1 (Thr t) s = s {thresh = t}
          p1 (Str t) s = s {string = t}
          p1 Dist s = s {distr = True}
          p1 Verb s = s {verb = True}
          p1 (Gap g) s = s {gap = g}
          p1 Server s = s {server = True}
          p1 (Library l) s = s {lib = Just l}
          p1 MaskLower s = s {lower = Lower}
          p1 NoStats s = s { stats = False }
          p1 (Skip i) s = s { skip = i }

\end{code}
